#! /usr/bin/env node
"use strict";
const ts = require("typescript");
const path = require("path");
const babel = require("babel-core");
const fs_extra_p_1 = require("fs-extra-p");
const bluebird_lst_c_1 = require("bluebird-lst-c");
const declarationGenerator_1 = require("./declarationGenerator");
const docGenerator_1 = require("./docGenerator");
const __awaiter = require("./awaiter");
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const paths = process.argv.slice(2);
        if (paths.length == 0) {
            paths.push(process.cwd());
        }
        for (let basePath of paths) {
            try {
                console.log(`Build ${basePath}`);
                yield build(basePath);
            }
            catch (e) {
                if (!(e instanceof CompilationError)) {
                    throw e;
                }
                for (let diagnostic of e.errors) {
                    if (diagnostic.file == null) {
                        console.log(diagnostic.messageText);
                        continue;
                    }
                    const location = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
                    const message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
                    console.log(`${diagnostic.file.fileName} (${location.line + 1}, ${location.character + 1}): ${message}`);
                }
                process.exit(-1);
                return;
            }
        }
    });
}
main()
    .catch(error => {
    console.error(error.stack || error.message || error);
    process.exit(-1);
});
function build(basePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const tsConfigPath = path.join(basePath, "tsconfig.json");
        const jsonResult = ts.parseConfigFileTextToJson(tsConfigPath, yield fs_extra_p_1.readFile(tsConfigPath, "utf8"));
        if (jsonResult.error != null) {
            throw new CompilationError([jsonResult.error]);
        }
        const result = ts.parseJsonConfigFileContent(jsonResult.config, ts.sys, basePath);
        checkErrors(result.errors);
        yield compile(basePath, result, jsonResult.config);
    });
}
function compile(basePath, config, tsConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        const compilerOptions = config.options;
        const declarationConfig = tsConfig.declaration;
        if (declarationConfig != null) {
            compilerOptions.declaration = true;
        }
        compilerOptions.noEmitOnError = true;
        const program = ts.createProgram(config.fileNames, compilerOptions, ts.createCompilerHost(compilerOptions));
        checkErrors(ts.getPreEmitDiagnostics(program));
        const outDir = compilerOptions.outDir;
        if (outDir == null) {
            throw new Error("outDir is not specified in the compilerOptions");
        }
        yield fs_extra_p_1.ensureDir(outDir);
        const target = compilerOptions.target || ts.ScriptTarget.Latest;
        const fileToSourceMap = {};
        const promises = [];
        const emittedFiles = new Set();
        const declarationFiles = Array();
        const emitResult = program.emit(undefined, (fileName, data) => {
            emittedFiles.add(fileName);
            if (fileName.endsWith(".js")) {
                const sourceMapFileName = fileName + ".map";
                processCompiled(data, fileToSourceMap[sourceMapFileName], fileName, sourceMapFileName, promises);
            }
            else if (fileName.endsWith(".js.map")) {
                fileToSourceMap[fileName] = data;
            }
            else if (declarationConfig != null) {
                declarationFiles.push(ts.createSourceFile(fileName, data, target, true));
            }
        });
        checkErrors(emitResult.diagnostics);
        if (emitResult.emitSkipped) {
            throw new Error("Emit skipped");
        }
        if (declarationFiles.length > 0) {
            let packageData = null;
            try {
                packageData = yield fs_extra_p_1.readJson(path.join(basePath, "package.json"));
            }
            catch (e) {
            }
            const main = packageData == null ? null : packageData.main;
            for (const moduleName of Object.keys(declarationConfig)) {
                promises.push(declarationGenerator_1.generateDeclarationFile(moduleName, declarationFiles, compilerOptions, path.join(basePath, declarationConfig[moduleName]), basePath, main));
            }
        }
        if (process.env.CI == null && tsConfig.docs != null) {
            const docs = docGenerator_1.generateDocs(program);
            if (docs.length !== 0) {
                promises.push(docGenerator_1.writeDocFile(path.resolve(basePath, tsConfig.docs), docs));
            }
        }
        yield bluebird_lst_c_1.default.all(promises);
        yield removeOld(outDir, emittedFiles);
    });
}
function removeOld(outDir, emittedFiles) {
    return __awaiter(this, void 0, void 0, function* () {
        const files = yield fs_extra_p_1.readdir(outDir);
        yield bluebird_lst_c_1.default.map(files, file => {
            if (file[0] !== "." && !file.endsWith(".d.ts") && file !== "__snapshots__") {
                const fullPath = `${outDir}/${file}`;
                if (!file.includes(".")) {
                    return removeOld(fullPath, emittedFiles);
                }
                else if (!emittedFiles.has(fullPath)) {
                    return fs_extra_p_1.unlink(fullPath);
                }
            }
            return null;
        });
    });
}
function processCompiled(code, sourceMap, jsFileName, sourceMapFileName, promises) {
    const result = babel.transform(code, {
        inputSourceMap: JSON.parse(sourceMap),
        sourceMaps: true,
        filename: jsFileName,
    });
    promises.push(fs_extra_p_1.outputFile(jsFileName, result.code), fs_extra_p_1.outputJson(sourceMapFileName, result.map));
}
function checkErrors(errors) {
    if (errors.length !== 0) {
        throw new CompilationError(errors);
    }
}
class CompilationError extends Error {
    constructor(errors) {
        super("Compilation error");
        this.errors = errors;
    }
}
//# sourceMappingURL=builder.js.map